==> mcss-0.0.43.tar.gz: mcss-0.0.43/mcss/McssProperties.cpp <==
// model properties simulation
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "McssProperties.h"
#include <string.h>
#include <math.h>

using namespace Simulation;

// find steady state species concentration
void Simulation::find_value_monotonic(McssPropertiesParameters \
		&properties_parameters, PropertiesResults &properties_results)
{
	double hi, lo, mid, ss_conc, tolerance, target, lo_min, hi_max;
	double max_time, steady_state_interval, log_interval;
	bool stochastic, flag, increasing, redo, show_progress, adjust_bounds;
	unsigned int iteration, max_iterations, comp_ind, input_ind;
	unsigned int num_output_species, *output_ind;
	unsigned int ss_timepoint, max_time_timepoint, timepoint;
	string::size_type index1, index2;
	
	// load mcss parameters
	SimulationParameters mcss_parameters;
	mcss_parameters.read(properties_parameters.mcss_parameters);
	
	// set search variables
	if(strcmp(mcss_parameters.simulation_algorithm, "ode1") == 0)
		stochastic = false;
	else
		stochastic = true;
	// log simulation to memory
	mcss_parameters.log_memory = true;
	// set to a single run
	mcss_parameters.runs = 1;
	max_time = mcss_parameters.max_time;
	log_interval = mcss_parameters.log_interval;
	steady_state_interval = properties_parameters.steady_state_interval;
	ss_timepoint = (unsigned int) ((double) (max_time - steady_state_interval) \
			/ log_interval);
	max_time_timepoint = (unsigned int) ((double) max_time / log_interval) - 1;
	max_iterations = properties_parameters.max_iterations;
	lo = properties_parameters.lower_bound;
	hi = properties_parameters.upper_bound;
	target = properties_parameters.target;
	tolerance = properties_parameters.tolerance;
	if(tolerance == -1.0) {
		tolerance = target - target * properties_parameters.accuracy;
		lo_min = lo * properties_parameters.accuracy;
		hi_max = hi * properties_parameters.accuracy;
	} else {
		lo_min = lo + tolerance;
		hi_max = hi - tolerance;
	}
	increasing = properties_parameters.increasing;
	show_progress = properties_parameters.show_progress;
	adjust_bounds = properties_parameters.adjust_bounds;
	// sanity check
	if(steady_state_interval >= max_time) {
		cerr << "error: steady_state_interval greater than max_time.\n";
		exit(EXIT_FAILURE);
	}
	
	// initialise model
	Psystem *psystem = new Psystem(mcss_parameters);	
	
	// find input and output species indices and compartments
	// compartment
	comp_ind = psystem->getCompartmentIndexFromName( \
			properties_parameters.compartment);
	if(comp_ind == psystem->getNumCompartments()) {
		cerr << "error: compartment " << properties_parameters.compartment << \
				" not found.\n";
		exit(EXIT_FAILURE);
	}
	// input species
	input_ind = psystem->getSpeciesIndex(properties_parameters.input_species);
	if(input_ind == psystem->getNumSpecies()) {
		cerr << "error: species " << properties_parameters.input_species << \
				" not found.\n";
		exit(EXIT_FAILURE);
	}
	// output species
	// find number of output species
	num_output_species = 1;
	index1 = 0;
	while((index1 = properties_parameters.output_species.find(',', index1)) \
				!= string::npos) {
		index1++;
		num_output_species++;
	}
	if(!num_output_species) {
		cerr << "error: no output species specified.\n";
		exit(EXIT_FAILURE);
	}
	// allocate memory for output species
	if(!(output_ind = (unsigned int *) calloc(num_output_species, \
			sizeof(unsigned int)))) {
		cerr << "error: unable to allocate memory for output species.\n";
		exit(EXIT_FAILURE);
	}
	// allocate memory to store outputs of individual species
	if(!(properties_results.outputs = (double *) calloc(num_output_species, \
			sizeof(double)))) {
		cerr << "error: unable to allocate memory for output species " \
				"results.\n";
		exit(EXIT_FAILURE);
	}	
	properties_results.num_output_species = num_output_species;
	// find output species indices
	index1 = 0;
	num_output_species = 0;
	do {
		index2 = properties_parameters.output_species.find(',', index1);
		if(index2 != string::npos)
			index2 -= index1;
		output_ind[num_output_species] = psystem->getSpeciesIndex( \
				properties_parameters.output_species.substr(index1, index2));
		properties_results.outputs[num_output_species] = -1.0;
		if(output_ind[num_output_species] == psystem->getNumSpecies()) {
			cerr << "error: species " << properties_parameters. \
					output_species.substr(index1, index2) << " not found.\n";
			exit(EXIT_FAILURE);
		}
		if(index2 != string::npos)
			index1 = index2 + 1;
		num_output_species++;
	} while(index2 != string::npos);
	
	iteration = 0;
	
	// set binary search bounds
	if(adjust_bounds) {
		// set upper bound
		flag = true;
		do {		
			// execute till steady state reached
			if(stochastic) {
				// steady state detection not supported for stochastic models
				// reset simulation
				psystem->resetSimulation();
				// set input species concentration
				psystem->setSpeciesConcentration(comp_ind, input_ind, hi);
				// execute model
				psystem->execute(max_time);
				iteration++;
			} else {
				redo = true;
				while(redo && iteration < max_iterations) {
					// reset simulation
					psystem->resetSimulation();
					// set input species concentration
					psystem->setSpeciesConcentration(comp_ind, input_ind, hi);
					// execute model
					psystem->execute(max_time);
					// check if reached steady state
					timepoint = ss_timepoint + 1;
					while(timepoint++ < max_time_timepoint) {
						for(index1 = 0; index1 < num_output_species; index1++)
							if(psystem->getSpeciesLevel(comp_ind, \
									output_ind[index1], timepoint) != \
									psystem->getSpeciesLevel(comp_ind, \
									output_ind[index1], timepoint - 1)) {
								redo = false;
								break;
							}
						if(!redo)
							break;
					}
					// not at steady state so increase max_time and repeat
					redo = !redo;
					if(redo) {
						max_time += steady_state_interval;
						ss_timepoint = (unsigned int) ((double) (max_time - \
								steady_state_interval) / log_interval);
						max_time_timepoint = (unsigned int) ((double) max_time \
								/ log_interval);
					} 				
					iteration++;
					if(show_progress && redo)
						printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
								iteration, hi, 0.0, lo, hi, max_time);
				}
				if(redo) {
					// unable to find upper bound
					delete psystem;
					properties_results.input = properties_results.output = -1.0;
					properties_results.max_time = max_time;
					properties_results.iterations = iteration;
					return;
				}
			}
			// find steady state concentration
			ss_conc = 0.0;
			for(index1 = 0; index1 < num_output_species; index1++)
				ss_conc += psystem->getSpeciesConcentration(comp_ind, \
						output_ind[index1]);
			if(show_progress)
				printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
						iteration, hi, ss_conc, lo, hi, max_time);
			if(increasing) {
				if(ss_conc < target) {
					// increase upper bound
					lo = hi;
					hi *= 10.0;
				} else
					flag = false;
			} else {
				if(ss_conc > target) {
					// increase upper bound
					lo = hi;
					hi *= 10.0;
				} else
					flag = false;
			}
		} while(flag && iteration < max_iterations);
		if(flag) {
			// unable to find upper bound
			delete psystem;
			properties_results.input = properties_results.output = -1.0;
			properties_results.max_time = max_time;
			properties_results.iterations = iteration;
			return;
		}
		if(iteration == 1) {
			// set lower bound
			flag = true;
			do {
				// execute till steady state reached
				if(stochastic) {
					// steady state detection not supported
					// reset simulation
					psystem->resetSimulation();
					// set input species concentration
					psystem->setSpeciesConcentration(comp_ind, input_ind, lo);
					// execute model
					psystem->execute(max_time);
					iteration++;
				} else {
					redo = true;
					while(redo && iteration < max_iterations) {
						// reset simulation
						psystem->resetSimulation();
						// set input species concentration
						psystem->setSpeciesConcentration(comp_ind, input_ind, \
								lo);
						// execute model
						psystem->execute(max_time);
						// check if reached steady state
						timepoint = ss_timepoint + 1;
						while(timepoint++ < max_time_timepoint) {
							for(index1 = 0; index1 < num_output_species; \
										index1++)
								if(psystem->getSpeciesLevel(comp_ind, \
										output_ind[index1], timepoint) != \
										psystem->getSpeciesLevel(comp_ind, \
										output_ind[index1], timepoint - 1)) {
									redo = false;
									break;
								}
							if(!redo)
								break;
						}
						// not at steady state so increase max_time and repeat
						redo = !redo;
						if(redo) {
							max_time += steady_state_interval;
							ss_timepoint = (unsigned int) ((double) \
									(max_time - steady_state_interval) \
									/ log_interval);
							max_time_timepoint = (unsigned int) \
									((double) max_time / log_interval);
						}					
						iteration++;
						if(show_progress && redo)
							printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
									iteration, lo, 0.0, lo, hi, max_time);
					}
					if(redo) {
						// unable to find lower bound
						delete psystem;
						properties_results.input = \
								properties_results.output = -1.0;
						properties_results.max_time = max_time;
						properties_results.iterations = iteration;
						return;
					}
				}
				// find steady state concentration
				ss_conc = 0.0;
				for(index1 = 0; index1 < num_output_species; index1++)
					ss_conc += psystem->getSpeciesConcentration(comp_ind, \
							output_ind[index1]);
				if(show_progress)
					printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
							iteration, lo, ss_conc, lo, hi, max_time);
				
				if(increasing) {
					if(ss_conc > target) {
						// decrease lower bound
						hi = lo;
						lo /= 10.0;
					} else
						flag = false;
				} else {
					if(ss_conc < target) {
						// decrease lower bound
						hi = lo;
						lo /= 10.0;
					} else
						flag = false;
				}
			} while(flag && iteration < max_iterations);
			if(flag) {
				// unable to find lower bound
				delete psystem;
				properties_results.input = properties_results.output = -1.0;
				properties_results.max_time = max_time;
				properties_results.iterations = iteration;
				return;
			}
		}
	}
	
	// perform binary search
	flag = true;
	do {
		mid = lo + (hi - lo) / 2.0;
		
		// execute till steady state reached
		if(stochastic) {
			// steady state detection not supported for stochastic models yet
			// reset simulation
			psystem->resetSimulation();
			// set input species concentration
			psystem->setSpeciesConcentration(comp_ind, input_ind, mid);
			// execute model
			psystem->execute(max_time);
			iteration++;
		} else {
			redo = true;
			while(redo && iteration < max_iterations) {
				// reset simulation
				psystem->resetSimulation();
				// set input species concentration
				psystem->setSpeciesConcentration(comp_ind, input_ind, mid);
				// execute model
				psystem->execute(max_time);
				// check if reached steady state
				timepoint = ss_timepoint + 1;
				while(timepoint++ < max_time_timepoint) {
					for(index1 = 0; index1 < num_output_species; index1++)
						if(psystem->getSpeciesLevel(comp_ind, \
								output_ind[index1], timepoint) != \
								psystem->getSpeciesLevel(comp_ind, \
								output_ind[index1], timepoint - 1)) {
							redo = false;
							break;
						}
					if(!redo)
						break;
				}
				// not at steady state so increase max_time and repeat
				redo = !redo;
				if(redo) {
					max_time += steady_state_interval;
					ss_timepoint = (unsigned int) ((double) (max_time - \
							steady_state_interval) / log_interval);
					max_time_timepoint = (unsigned int) ((double) max_time \
							/ log_interval);
				}				
				iteration++;
				if(show_progress && redo)
					printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
							iteration, mid, ss_conc, lo, hi, max_time);
			}
			if(redo) {
				// unable to find target
				delete psystem;
				properties_results.input = properties_results.output = -1.0;
				properties_results.max_time = max_time;
				properties_results.iterations = iteration;
				return;
			}
		}
		// find steady state concentration
		ss_conc = 0.0;
		for(index1 = 0; index1 < num_output_species; index1++)
			ss_conc += psystem->getSpeciesConcentration(comp_ind, \
					output_ind[index1]);
		if(show_progress)
			printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
				iteration, mid, ss_conc, lo, hi, max_time);
				
		if(fabs(target - ss_conc) <= tolerance)
			flag = false;
		else {
			// adjust search range
			if(increasing) {
				if(ss_conc < target)
					lo = mid;
				else
					hi = mid;
			} else {
				if(ss_conc < target)
					hi = mid;
				else	
					lo = mid;
			}
		}
		if(!adjust_bounds)
			if(mid < lo_min || mid > hi_max) {
				// unable to find target
				delete psystem;
				properties_results.input = properties_results.output = -1.0;
				properties_results.max_time = max_time;
				properties_results.iterations = iteration;
				return;
			}
	} while(flag && iteration < max_iterations);
	if(flag) {
		// unable to find target
		delete psystem;
		properties_results.input = properties_results.output = -1.0;
		properties_results.max_time = max_time;
		properties_results.iterations = iteration;
		return;
	}
	
	// found target within specified tolerance
	properties_results.input = (double) mid;
	properties_results.output = (double) ss_conc;
	properties_results.max_time = max_time;
	properties_results.iterations = iteration;
	for(index1 = 0; index1 < num_output_species; index1++)
		properties_results.outputs[index1] = \
				psystem->getSpeciesConcentration(comp_ind, output_ind[index1]);

	// tidy up
	delete psystem;
}

==> mcss-1.0.1.tar.gz: mcss-1.0.1/mcss/McssProperties.cpp <==
// model properties simulation
// copyright 2008, 2009 jamie twycross, jpt AT cs.nott.ac.uk
// released under GNU GPL version 3

#if HAVE_CONFIG_H
	#include <config.h>
#endif

#include "McssProperties.h"
#include <string.h>
#include <math.h>

using namespace Simulation;

// find steady state species concentration
void Simulation::find_value_monotonic(McssPropertiesParameters \
		&properties_parameters, PropertiesResults &properties_results)
{
	double hi, lo, mid, ss_conc, tolerance, target, lo_min, hi_max;
	double max_time, steady_state_interval, log_interval;
	bool stochastic, flag, increasing, redo, show_progress, adjust_bounds;
	unsigned int iteration, max_iterations, comp_ind, input_ind;
	unsigned int num_output_species, *output_ind;
	unsigned int ss_timepoint, max_time_timepoint, timepoint;
	string::size_type index1, index2;
	
	// load mcss parameters
	SimulationParameters mcss_parameters;
	mcss_parameters.read(properties_parameters.mcss_parameters);
	
	// set search variables
	if(strcmp(mcss_parameters.simulation_algorithm, "ode1") == 0)
		stochastic = false;
	else
		stochastic = true;
	// log simulation to memory
	mcss_parameters.log_memory = true;
	// set to a single run
	mcss_parameters.runs = 1;
	max_time = mcss_parameters.max_time;
	log_interval = mcss_parameters.log_interval;
	steady_state_interval = properties_parameters.steady_state_interval;
	ss_timepoint = (unsigned int) ((double) (max_time - steady_state_interval) \
			/ log_interval);
	max_time_timepoint = (unsigned int) ((double) max_time / log_interval) - 1;
	max_iterations = properties_parameters.max_iterations;
	lo = properties_parameters.lower_bound;
	hi = properties_parameters.upper_bound;
	target = properties_parameters.target;
	tolerance = properties_parameters.tolerance;
	if(tolerance == -1.0) {
		tolerance = target - target * properties_parameters.accuracy;
		lo_min = lo * properties_parameters.accuracy;
		hi_max = hi * properties_parameters.accuracy;
	} else {
		lo_min = lo + tolerance;
		hi_max = hi - tolerance;
	}
	increasing = properties_parameters.increasing;
	show_progress = properties_parameters.show_progress;
	adjust_bounds = properties_parameters.adjust_bounds;
	// sanity check
	if(steady_state_interval >= max_time) {
		cerr << "error: steady_state_interval greater than max_time.\n";
		exit(EXIT_FAILURE);
	}
	properties_results.input = -1.0;
	properties_results.input_ss = -1.0;
	properties_results.output = -1.0;
	
	// initialise model
	Psystem *psystem = new Psystem(mcss_parameters);	
	
	// find input and output species indices and compartments
	// compartment
	comp_ind = psystem->getCompartmentIndexFromName( \
			properties_parameters.compartment);
	if(comp_ind == psystem->getNumCompartments()) {
		cerr << "error: compartment " << properties_parameters.compartment << \
				" not found.\n";
		exit(EXIT_FAILURE);
	}
	// input species
	input_ind = psystem->getSpeciesIndex(properties_parameters.input_species);
	if(input_ind == psystem->getNumSpecies()) {
		cerr << "error: species " << properties_parameters.input_species << \
				" not found.\n";
		exit(EXIT_FAILURE);
	}
	// output species
	// find number of output species
	num_output_species = 1;
	index1 = 0;
	while((index1 = properties_parameters.output_species.find(',', index1)) \
				!= string::npos) {
		index1++;
		num_output_species++;
	}
	if(!num_output_species) {
		cerr << "error: no output species specified.\n";
		exit(EXIT_FAILURE);
	}
	// allocate memory for output species
	if(!(output_ind = (unsigned int *) calloc(num_output_species, \
			sizeof(unsigned int)))) {
		cerr << "error: unable to allocate memory for output species.\n";
		exit(EXIT_FAILURE);
	}
	// allocate memory to store outputs of individual species
	if(!(properties_results.outputs = (double *) calloc(num_output_species, \
			sizeof(double)))) {
		cerr << "error: unable to allocate memory for output species " \
				"results.\n";
		exit(EXIT_FAILURE);
	}	
	properties_results.num_output_species = num_output_species;
	// find output species indices
	index1 = 0;
	num_output_species = 0;
	do {
		index2 = properties_parameters.output_species.find(',', index1);
		if(index2 != string::npos)
			index2 -= index1;
		output_ind[num_output_species] = psystem->getSpeciesIndex( \
				properties_parameters.output_species.substr(index1, index2));
		properties_results.outputs[num_output_species] = -1.0;
		if(output_ind[num_output_species] == psystem->getNumSpecies()) {
			cerr << "error: species " << properties_parameters. \
					output_species.substr(index1, index2) << " not found.\n";
			exit(EXIT_FAILURE);
		}
		if(index2 != string::npos)
			index1 = index2 + 1;
		num_output_species++;
	} while(index2 != string::npos);
	
	iteration = 0;
	
	// set binary search bounds
	if(adjust_bounds) {
		// set upper bound
		flag = true;
		do {		
			// execute till steady state reached
			if(stochastic) {
				// steady state detection not supported for stochastic models
				// reset simulation
				psystem->resetSimulation();
				// set input species concentration
				psystem->setSpeciesConcentration(comp_ind, input_ind, hi);
				// execute model
				psystem->execute(max_time);
				iteration++;
			} else {
				redo = true;
				while(redo && iteration < max_iterations) {
					// reset simulation
					psystem->resetSimulation();
					// set input species concentration
					psystem->setSpeciesConcentration(comp_ind, input_ind, hi);
					// execute model
					psystem->execute(max_time);
					// check if reached steady state
					timepoint = ss_timepoint + 1;
					while(timepoint++ < max_time_timepoint) {
						for(index1 = 0; index1 < num_output_species; index1++)
							if(psystem->getSpeciesLevel(comp_ind, \
									output_ind[index1], timepoint) != \
									psystem->getSpeciesLevel(comp_ind, \
									output_ind[index1], timepoint - 1)) {
								redo = false;
								break;
							}
						if(!redo)
							break;
					}
					// not at steady state so increase max_time and repeat
					redo = !redo;
					if(redo) {
						max_time += steady_state_interval;
						ss_timepoint = (unsigned int) ((double) (max_time - \
								steady_state_interval) / log_interval);
						max_time_timepoint = (unsigned int) ((double) max_time \
								/ log_interval);
					} 				
					iteration++;
					if(show_progress && redo)
						printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
								iteration, hi, 0.0, lo, hi, max_time);
				}
				if(redo) {
					// unable to find upper bound
					delete psystem;
					properties_results.max_time = max_time;
					properties_results.iterations = iteration;
					return;
				}
			}
			// find steady state concentration
			ss_conc = 0.0;
			for(index1 = 0; index1 < num_output_species; index1++)
				ss_conc += psystem->getSpeciesConcentration(comp_ind, \
						output_ind[index1]);
			if(show_progress)
				printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
						iteration, hi, ss_conc, lo, hi, max_time);
			if(increasing) {
				if(ss_conc < target) {
					// increase upper bound
					lo = hi;
					hi *= 10.0;
				} else
					flag = false;
			} else {
				if(ss_conc > target) {
					// increase upper bound
					lo = hi;
					hi *= 10.0;
				} else
					flag = false;
			}
		} while(flag && iteration < max_iterations);
		if(flag) {
			// unable to find upper bound
			delete psystem;
			properties_results.max_time = max_time;
			properties_results.iterations = iteration;
			return;
		}
		if(iteration == 1) {
			// set lower bound
			flag = true;
			do {
				// execute till steady state reached
				if(stochastic) {
					// steady state detection not supported
					// reset simulation
					psystem->resetSimulation();
					// set input species concentration
					psystem->setSpeciesConcentration(comp_ind, input_ind, lo);
					// execute model
					psystem->execute(max_time);
					iteration++;
				} else {
					redo = true;
					while(redo && iteration < max_iterations) {
						// reset simulation
						psystem->resetSimulation();
						// set input species concentration
						psystem->setSpeciesConcentration(comp_ind, input_ind, \
								lo);
						// execute model
						psystem->execute(max_time);
						// check if reached steady state
						timepoint = ss_timepoint + 1;
						while(timepoint++ < max_time_timepoint) {
							for(index1 = 0; index1 < num_output_species; \
										index1++)
								if(psystem->getSpeciesLevel(comp_ind, \
										output_ind[index1], timepoint) != \
										psystem->getSpeciesLevel(comp_ind, \
										output_ind[index1], timepoint - 1)) {
									redo = false;
									break;
								}
							if(!redo)
								break;
						}
						// not at steady state so increase max_time and repeat
						redo = !redo;
						if(redo) {
							max_time += steady_state_interval;
							ss_timepoint = (unsigned int) ((double) \
									(max_time - steady_state_interval) \
									/ log_interval);
							max_time_timepoint = (unsigned int) \
									((double) max_time / log_interval);
						}					
						iteration++;
						if(show_progress && redo)
							printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
									iteration, lo, 0.0, lo, hi, max_time);
					}
					if(redo) {
						// unable to find lower bound
						delete psystem;
						properties_results.max_time = max_time;
						properties_results.iterations = iteration;
						return;
					}
				}
				// find steady state concentration
				ss_conc = 0.0;
				for(index1 = 0; index1 < num_output_species; index1++)
					ss_conc += psystem->getSpeciesConcentration(comp_ind, \
							output_ind[index1]);
				if(show_progress)
					printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
							iteration, lo, ss_conc, lo, hi, max_time);
				
				if(increasing) {
					if(ss_conc > target) {
						// decrease lower bound
						hi = lo;
						lo /= 10.0;
					} else
						flag = false;
				} else {
					if(ss_conc < target) {
						// decrease lower bound
						hi = lo;
						lo /= 10.0;
					} else
						flag = false;
				}
			} while(flag && iteration < max_iterations);
			if(flag) {
				// unable to find lower bound
				delete psystem;
				properties_results.max_time = max_time;
				properties_results.iterations = iteration;
				return;
			}
		}
	}
	
	// perform binary search
	flag = true;
	do {
		mid = lo + (hi - lo) / 2.0;
		
		// execute till steady state reached
		if(stochastic) {
			// steady state detection not supported for stochastic models yet
			// reset simulation
			psystem->resetSimulation();
			// set input species concentration
			psystem->setSpeciesConcentration(comp_ind, input_ind, mid);
			// execute model
			psystem->execute(max_time);
			iteration++;
		} else {
			redo = true;
			while(redo && iteration < max_iterations) {
				// reset simulation
				psystem->resetSimulation();
				// set input species concentration
				psystem->setSpeciesConcentration(comp_ind, input_ind, mid);
				// execute model
				psystem->execute(max_time);
				// check if reached steady state
				timepoint = ss_timepoint + 1;
				while(timepoint++ < max_time_timepoint) {
					for(index1 = 0; index1 < num_output_species; index1++)
						if(psystem->getSpeciesLevel(comp_ind, \
								output_ind[index1], timepoint) != \
								psystem->getSpeciesLevel(comp_ind, \
								output_ind[index1], timepoint - 1)) {
							redo = false;
							break;
						}
					if(!redo)
						break;
				}
				// not at steady state so increase max_time and repeat
				redo = !redo;
				if(redo) {
					max_time += steady_state_interval;
					ss_timepoint = (unsigned int) ((double) (max_time - \
							steady_state_interval) / log_interval);
					max_time_timepoint = (unsigned int) ((double) max_time \
							/ log_interval);
				}				
				iteration++;
				if(show_progress && redo)
					printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
							iteration, mid, ss_conc, lo, hi, max_time);
			}
			if(redo) {
				// unable to find target
				delete psystem;
				properties_results.max_time = max_time;
				properties_results.iterations = iteration;
				return;
			}
		}
		// find steady state concentration
		ss_conc = 0.0;
		for(index1 = 0; index1 < num_output_species; index1++)
			ss_conc += psystem->getSpeciesConcentration(comp_ind, \
					output_ind[index1]);
		if(show_progress)
			printf("%d %0.8E %0.8E %0.8E %0.8E %0.2f\n", \
				iteration, mid, ss_conc, lo, hi, max_time);
				
		if(fabs(target - ss_conc) <= tolerance)
			flag = false;
		else {
			// adjust search range
			if(increasing) {
				if(ss_conc < target)
					lo = mid;
				else
					hi = mid;
			} else {
				if(ss_conc < target)
					hi = mid;
				else	
					lo = mid;
			}
		}
		if(!adjust_bounds)
			if(mid < lo_min || mid > hi_max) {
				// unable to find target
				delete psystem;
				properties_results.max_time = max_time;
				properties_results.iterations = iteration;
				return;
			}
	} while(flag && iteration < max_iterations);
	if(flag) {
		// unable to find target
		delete psystem;
		properties_results.max_time = max_time;
		properties_results.iterations = iteration;
		return;
	}
	
	// found target within specified tolerance
	properties_results.input = (double) mid;
	properties_results.input_ss = psystem->getSpeciesConcentration(comp_ind, \
			input_ind);
	properties_results.output = (double) ss_conc;
	properties_results.max_time = max_time;
	properties_results.iterations = iteration;
	for(index1 = 0; index1 < num_output_species; index1++)
		properties_results.outputs[index1] = \
				psystem->getSpeciesConcentration(comp_ind, output_ind[index1]);

	// tidy up
	delete psystem;
}
